subgraph cluster_{{ name }} {
    color = "{{ border_color }}"
    margin={{ margin }}
    style="dashed"
    label = "  {{ label }}"
    labeljust = "l"
    fontsize = {{ fontsize }}
    {{ content }}
}
