/*
 * Copyright (c) 2015 Mountainstorm
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/* this element needs tooltip positioning to work */
.graphviz-svg {
  position: relative;
}

/* stop tooltips wrapping */
.graphviz-svg .tooltip-inner {
  white-space: nowrap;
}

/* stop people selecting text on nodes */
.graphviz-svg text {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -khtml-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
  cursor: default;
}

/* ==================== */
/* Magnifying Glass Styles */
/* ==================== */

.magnifying-lens {
  pointer-events: none;
  z-index: 9999;
}

.magnifying-lens .lens-border {
  filter: drop-shadow(0 4px 8px rgba(0, 0, 0, 0.4));
}

.magnifying-lens .lens-content {
  shape-rendering: geometricPrecision;
  text-rendering: geometricPrecision;
}

svg.magnifier-active {
  cursor: none;
}
