<tr><td cellpadding="6" bgcolor="{{ bg_color }}" align="center" colspan="1" width="75" {% if port %}port="{{ port }}"{% endif %}><font color="white">{% if is_entity %}<b>{{ text }} (E)</b>{% else %}{{ text }}{% endif %}</font></td></tr>
